@@ -4,9 +4,26 @@ from PyQt5.QtWidgets import QLabel
 from feeluown.utils.utils import parse_ms
 
 
+def format_second(s):
+    """Format mileseconds to text
+
+    >>> format_second(1)
+    '00:01'
+    """
+    m, s = parse_ms(s * 1000)
+    h = m // 60
+    m = m % 60
+    t = QTime(h, m, s)
+    if h > 0:
+        text = t.toString('hh:mm:ss')
+    else:
+        text = t.toString('mm:ss')
+    return text
+
+
 class DurationLabel(QLabel):
     def __init__(self, app, parent=None):
-        super().__init__('00：00', parent=parent)
+        super().__init__('00:00', parent=parent)
         self._app = app
 
         self.setAlignment(Qt.AlignCenter)
@@ -14,25 +31,16 @@ class DurationLabel(QLabel):
             self.on_duration_changed, aioqueue=True)
 
     def on_duration_changed(self, duration):
-        duration = duration * 1000
-        m, s = parse_ms(duration)
-        t = QTime(0, m, s)
-        self.setText(t.toString('mm:ss'))
+        self.setText(format_second(duration or 0))
 
 
 class ProgressLabel(QLabel):
     def __init__(self, app, parent=None):
-        super().__init__('00：00', parent=parent)
+        super().__init__('00:00', parent=parent)
         self._app = app
 
         self.setAlignment(Qt.AlignCenter)
-        self._app.player.position_changed.connect(
-            self.on_position_changed, aioqueue=True)
+        self._app.player_pos_per300ms.changed.connect(self.on_position_changed)
 
     def on_position_changed(self, position):
-        if position is None:
-            return
-        position = position * 1000
-        m, s = parse_ms(position)
-        t = QTime(0, m, s)
-        self.setText(t.toString('mm:ss'))
+        self.setText(format_second(position or 0))
