@@ -8,11 +8,11 @@ from PyQt5.QtCore import (
     QAbstractTableModel, QAbstractListModel, QModelIndex,
     QSize, QRect, QPoint, QPointF, QSortFilterProxyModel,
 )
-from PyQt5.QtGui import QPainter, QPalette, QPen, QMouseEvent, QPolygonF
+from PyQt5.QtGui import QPainter, QPalette, QMouseEvent, QPolygonF
 from PyQt5.QtWidgets import (
     QAction, QFrame, QHBoxLayout, QAbstractItemView, QHeaderView,
     QPushButton, QTableView, QWidget, QMenu, QListView,
-    QStyle, QSizePolicy, QStyledItemDelegate,
+    QStyle, QSizePolicy, QStyledItemDelegate
 )
 
 from feeluown.utils import aio
@@ -182,7 +182,9 @@ class SongListDelegate(QStyledItemDelegate):
                     self.play_btn_pressed = True
                 if event.type() == QEvent.MouseButtonRelease:
                     if self.play_btn_pressed is True:
-                        self.parent().play_song_needed.emit(index.data(Qt.UserRole))
+                        parent = self.parent()
+                        assert isinstance(parent, SongListView)
+                        parent.play_song_needed.emit(index.data(Qt.UserRole))
             if event.type() == QEvent.MouseButtonRelease:
                 self.play_btn_pressed = False
         return super().editorEvent(event, model, option, index)
@@ -199,8 +201,7 @@ class SongListView(ItemViewNoScrollMixin, QListView):
     play_song_needed = pyqtSignal([object])
 
     def __init__(self, parent=None, **kwargs):
-        super().__init__(parent, **kwargs)
-        QListView.__init__(self, parent)
+        super().__init__(parent=parent, **kwargs)
 
         self.delegate = SongListDelegate(self)
         self.setItemDelegate(self.delegate)
@@ -239,8 +240,8 @@ class BaseSongsTableModel(QAbstractTableModel):
 
     def flags(self, index):
         # Qt.NoItemFlags is ItemFlag and we should return ItemFlags
-        no_item_flags = Qt.ItemIsSelectable
-        if index.column() in (Column.source, Column.index, Column.duration):
+        no_item_flags = Qt.ItemIsSelectable | Qt.ItemIsEnabled
+        if index.column() in (Column.index, Column.source, Column.duration):
             return no_item_flags
 
         # Default flags.
@@ -280,7 +281,9 @@ class BaseSongsTableModel(QAbstractTableModel):
                 # we set height to 25 since the header can be short under macOS.
                 # HELP: set height to fixed value manually is not so elegant
                 height = 25
-                w = self.columns_config.get_width(section, self.parent().width())
+                parent = self.parent()
+                assert isinstance(parent, QWidget)
+                w = self.columns_config.get_width(section, parent.width())
                 return QSize(w, height)
         else:
             if role == Qt.DisplayRole:
@@ -467,27 +470,29 @@ class SongsTableDelegate(QStyledItemDelegate):
 
     def paint(self, painter, option, index):
         super().paint(painter, option, index)
-
         painter.setRenderHint(QPainter.Antialiasing)
         hovered = index.row() == self.row_hovered
 
         # Draw play button on Column.index when the row is hovered.
         if hovered and index.column() == Column.index:
             painter.save()
-            # Override contents.
-            if option.state & QStyle.State_Selected:  # type: ignore
-                bgcolor = option.palette.color(QPalette.Active, QPalette.Highlight)
-                fgcolor = option.palette.color(QPalette.Active, QPalette.HighlightedText)
-            else:
-                bgcolor = option.palette.color(QPalette.Active, QPalette.Base)
-                fgcolor = option.palette.color(QPalette.Active, QPalette.Text)
-            painter.setBrush(bgcolor)
+            # Override the content drawed by super().paint.
             painter.setPen(Qt.NoPen)
+            # HELP(cosven): when an item was hovered, super().paint may draw
+            # a semi-transparent rect over the item or draw a different color
+            # for the text. The rect/text color may not be in the palette.
+            # I checked the qt source code, and I think this behaviour is
+            # platform indenpent.It is drawed by something like KDE, kvantum.
+            # We have no way to draw a similar look (or please help find a way).
+            if index.row() % 2 == 0:
+                painter.setBrush(option.palette.color(QPalette.Base))
+            else:
+                painter.setBrush(option.palette.color(QPalette.AlternateBase))
             painter.drawRect(option.rect)
             # Draw play button.
+            painter.setBrush(option.palette.color(QPalette.Text))
             triangle_edge = 12
             triangle_height = 10
-            painter.setBrush(fgcolor)
             # Move the triangle right 2px and it looks better.
             painter.translate(
                 2 + option.rect.x() + (option.rect.width() - triangle_height)//2,
@@ -499,23 +504,7 @@ class SongsTableDelegate(QStyledItemDelegate):
             painter.drawPolygon(triangle)
             painter.restore()
 
-        # Draw a line under each row. If it is hovered, highlight the line.
-        painter.save()
-        pen = QPen()
-        line_color = option.palette.color(QPalette.Active, QPalette.Text)
-        line_color.setAlpha(30)
-        pen.setColor(line_color)
-        painter.setPen(pen)
-        bottom_left = option.rect.bottomLeft()
-        bottom_right = option.rect.bottomRight()
-        if index.model().columnCount() - 1 == index.column():
-            bottom_right = QPoint(bottom_right.x(), bottom_right.y())
-        if index.column() == 0:
-            bottom_left = QPoint(bottom_left.x(), bottom_right.y())
-        painter.drawLine(bottom_left, bottom_right)
-        painter.restore()
-
-        # Draw the mask over the row.
+        # Since the selection behaviour is SelectRows, so draw the mask over the row.
         if hovered:
             painter.save()
             mask_color = option.palette.color(QPalette.Active, QPalette.Text)
@@ -534,8 +523,10 @@ class SongsTableDelegate(QStyledItemDelegate):
         """
         if index.isValid() and self.parent() is not None:
             # The way getting the sourceModel seems a little strange.
+            parent = self.parent()
+            assert isinstance(parent, QWidget)
             w = index.model().sourceModel().columns_config.get_width(
-                index.column(), self.parent().width())
+                index.column(), parent.width())
             h = option.rect.height()
             return QSize(w, h)
         return super().sizeHint(option, index)
@@ -548,7 +539,9 @@ class SongsTableDelegate(QStyledItemDelegate):
                 self.pressed_cell = cell
             elif etype == QEvent.MouseButtonRelease:
                 if cell == self.pressed_cell and cell[1] == Column.index:
-                    self.parent().play_song_needed.emit(index.data(Qt.UserRole))
+                    parent = self.parent()
+                    assert isinstance(parent, SongsTableView)
+                    parent.play_song_needed.emit(index.data(Qt.UserRole))
                 self.pressed_cell = None
 
         return super().editorEvent(event, model, option, index)
@@ -569,14 +562,12 @@ class SongsTableView(ItemViewNoScrollMixin, QTableView):
     row_hovered = pyqtSignal([object])  # None when not hovered, row id when hovered.
 
     def __init__(self, app, parent=None):
-        super().__init__(parent)
-        QTableView.__init__(self, parent)
+        super().__init__(parent=parent)
 
         self._app = app
 
         # override ItemViewNoScrollMixin variables
         self._least_row_count = 6
-        self._row_height = 40
 
         # slot functions
         self.remove_song_func = None
@@ -594,9 +585,9 @@ class SongsTableView(ItemViewNoScrollMixin, QTableView):
         self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         self.setFrameShape(QFrame.NoFrame)
+        self.setAlternatingRowColors(True)
         self.verticalHeader().hide()
         self.horizontalHeader().hide()
-        self.verticalHeader().setDefaultSectionSize(self._row_height)
         self.setWordWrap(False)
         self.setTextElideMode(Qt.ElideRight)
         self.setMouseTracking(True)
@@ -627,9 +618,14 @@ class SongsTableView(ItemViewNoScrollMixin, QTableView):
             hide_columns = [Column.source]
             columns_config.set_width_ratio(Column.artist, 0.2)
             columns_config.set_width_ratio(Column.album, 0.3)
-        self.model().sourceModel().update_columns_config(columns_config)
 
-        for i in range(0, self.model().columnCount()):
+        model = self.model()
+        assert isinstance(model, SongFilterProxyModel)
+        source = model.sourceModel()
+        assert isinstance(source, SongsTableModel)
+        source.update_columns_config(columns_config)
+
+        for i in range(0, model.columnCount()):
             if i in hide_columns:
                 self.hideColumn(i)
             else:
@@ -693,6 +689,7 @@ class SongsTableView(ItemViewNoScrollMixin, QTableView):
             if song not in songs_to_remove:
                 songs_to_remove.append(song)
         for song in songs_to_remove:
+            assert callable(self.remove_song_func)
             self.remove_song_func(song)
 
     def viewportEvent(self, event):
